import sinon from 'sinon';
import assert from 'assert';
import { ZMQ_PUB_CHANNELS, ZMQ_SUB_CHANNELS } from '../../lib/system/constants.mjs';
import { ZmqObserver, ZmqPublisher } from '../utils/index.mjs';
import DbSeeds from '../seeds/data.json' assert { type: 'json' };
import { Server, createEvent, initRepoData, clearRepoData, findNodeByMac, delay, deleteNodeByMac } from './helpers.mjs';


describe('Handle ~event message and retranslate to $event', function () {
    // ~pos and $pos descriptions: https://redpointpositioning.atlassian.net/wiki/spaces/REDPOINT/pages/1314031083/Platform+Message+Processor#Handling-%E2%80%9Cpos%E2%80%9D-messages
    const zmqObserver = new ZmqObserver();
    const zmqPublisher = new ZmqPublisher();

    before(async function () {
        await Server().init();
        await initRepoData();
        zmqObserver.init();
        await zmqPublisher.init(); // init with some delay to init zmq connection
    });
    after(async function () {
        await clearRepoData();
        zmqObserver.destroy();
        zmqPublisher.destroy();
    });
    it('should extend event from the JSON/BIN convertor with trackable/sublocation details', function (done) {
        const association = DbSeeds.associations[0];
        const sublocation = DbSeeds.sublocations[0];
        const event = createEvent('heartbeat', association.mac, sublocation.id);
        const subscription = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.EVENT, function (zmqMsg, channel) {
            // console.log("ZmqMsg: ", zmqMsg);
            assert(ZMQ_PUB_CHANNELS.EVENT === channel, `Message channel not equal to ${ZMQ_PUB_CHANNELS.EVENT}`);
            assert(zmqMsg.ts === event.ts, `Event timestamp not equal to ${event.ts}`);
            assert(zmqMsg.type === event.type, `Event type not equal to ${event.ts}`);
            assert(zmqMsg.data.sl_uid === sublocation.uid, `Sublocation uid is not valid`);

            done();
        });
        subscription.then(() => {
            zmqPublisher.sendToMp(ZMQ_SUB_CHANNELS.EVENT, event);
        });
    });
    it('should generate pos on trackable event', function (done) {
        const association = DbSeeds.associations[0];
        const sublocation = DbSeeds.sublocations[0];
        const event = createEvent('heartbeat', association.mac, sublocation.id);
        let handlePos = false;
        let handleEvent = false;
        const subscriptionPos = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.POS, function (zmqMsg, channel) {
            // console.log("ZmqMsg: ", zmqMsg);
            assert(ZMQ_PUB_CHANNELS.POS === channel, `Message channel not equal to ${ZMQ_PUB_CHANNELS.POS}`);
            assert(zmqMsg.tk_uid === association.trackable_uid, `Trackable uid is not identified`);
            assert(zmqMsg.ts === event.ts, `Position timestamp not equal to ${event.ts}`);
            assert(zmqMsg.data.mac === event.mac, `Position timestamp not equal to ${event.mac}`);

            assert(zmqMsg.data.x === event.data.x, `Position X not equal to ${event.data.x}`);
            assert(zmqMsg.data.y === event.data.y, `Position Y not equal to ${event.data.y}`);
            assert(zmqMsg.data.z === event.data.z, `Position Z not equal to ${event.data.z}`);
            handlePos = true;
        })
        const subscriptionEvent = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.EVENT, function (zmqMsg, channel) {
            assert(ZMQ_PUB_CHANNELS.EVENT === channel, `Message channel not equal to ${ZMQ_PUB_CHANNELS.EVENT}`);
            handleEvent = true;
        });
        Promise.all([subscriptionPos, subscriptionEvent]).then(() => {
            zmqPublisher.sendToMp(ZMQ_SUB_CHANNELS.EVENT, event);
            return delay(100);
        }).then(() => {
            assert(handleEvent);
            assert(handlePos);
            done();
        });
    });
    
    it('should not generate pos on trackable event when no sublocation_uid.', function (done) {
        const association = DbSeeds.associations[0];
        const sublocation = DbSeeds.sublocations[0];
        let handleEvent = false;
        
        const event = createEvent('heartbeat', association.mac, sublocation.id);
        delete event.data.sublocation_id;
        const spy = sinon.spy();
        const subscriptionPos = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.POS, spy);
    
        const subscriptionEvent = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.EVENT, function (zmqMsg, channel) {
            assert(ZMQ_PUB_CHANNELS.EVENT === channel, `Message channel not equal to ${ZMQ_PUB_CHANNELS.EVENT}`);
            handleEvent = true;
        });
        
        Promise.all([subscriptionPos, subscriptionEvent]).then(() => {
            zmqPublisher.sendToMp(ZMQ_SUB_CHANNELS.EVENT, event);
            return delay(100);
        }).then(() => {
            assert(spy.callCount === 0);
            assert(handleEvent);
            done();
        });
    });

    it('should not generate pos on trackable event when wrong sublocation_uid.', function (done) {
        const association = DbSeeds.associations[0];
        const sublocation = DbSeeds.sublocations[0];
        let handleEvent = false;
        
        const event = createEvent('heartbeat', association.mac, sublocation.id);
        event.data.sublocation_id = 256;
        const spy = sinon.spy();
        const subscriptionPos = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.POS, spy);
    
        const subscriptionEvent = zmqObserver.subscribe(ZMQ_PUB_CHANNELS.EVENT, function (zmqMsg, channel) {
            assert(ZMQ_PUB_CHANNELS.EVENT === channel, `Message channel not equal to ${ZMQ_PUB_CHANNELS.EVENT}`);
            handleEvent = true;
        });
        
        Promise.all([subscriptionPos, subscriptionEvent]).then(() => {
            zmqPublisher.sendToMp(ZMQ_SUB_CHANNELS.EVENT, event);
            return delay(100);
        }).then(() => {
            assert(spy.callCount === 0);
            assert(handleEvent);
            done();
        });
    });

    it('should auto create device', async function () {
        const notExistingMac = '010101010101';

        const event = createEvent('announce', notExistingMac);

        await zmqPublisher.sendToMp(ZMQ_SUB_CHANNELS.EVENT, event);

        await delay(1000);
        const node = await findNodeByMac(notExistingMac);
        assert(node.mac === notExistingMac, 'Fail to auto create device');
        await deleteNodeByMac(node);
    });
});
