import assert from 'assert';
import { UID } from '../../lib/utils/uid.mjs';

import { post, put, del } from '../utils/index.mjs';
import { createTrackable, createTag, Server } from './helpers.mjs';

describe('Concurrent Associations Management', function () {
    const tagAssociationOne = {};
    const tagAssociationTwo = {};
    const trackableOne = {};
    const trackableTwo = {};
    const tagOne = {};
    const tagTwo = {};

    before(async function () {
        await Server().init();
        Object.assign(trackableOne, await createTrackable(`Trackable One ${UID()}`));
        Object.assign(trackableTwo, await createTrackable(`Trackable Two ${UID()}`));
        Object.assign(tagOne, await createTag('04956EA5010A'));
        Object.assign(tagTwo, await createTag('04956EA5010B'));
    });

    after(async function () {
        await del(`/api/v3/trackables/${trackableOne.uid}`);
        await del(`/api/v3/trackables/${trackableTwo.uid}`);
        await del(`/api/v3/tags/${tagOne.mac}`);
        await del(`/api/v3/tags/${tagTwo.mac}`);
    });

    describe('Create Two Tag Association', function () {
        it('must create tag association', async function () {
            let bodyOne = {
                mac: tagOne.mac.toLowerCase(),
                trackable_uid: trackableOne.uid,
            };

            let bodyTwo = {
                mac: tagTwo.mac.toLowerCase(),
                trackable_uid: trackableTwo.uid,
            };

            const oddPair1 = {
                uid: trackableOne.uid,
                mac: '04956EA5010A',
            };
            const evenPair1 = {
                uid: trackableOne.uid,
                mac: '04956EA5010B',
            };

            const oddPair2 = {
                uid: trackableTwo.uid,
                mac: '04956EA5010B',
            };
            const evenPair2 = {
                uid: trackableTwo.uid,
                mac: '04956EA5010A',
            };

            let pairOne = oddPair1;
            let pairTwo = oddPair2;

            const promise1 = new Promise((resolve, reject) => {
                post('/api/v3/tag_assignments', bodyOne)
                    .expect(200)
                    .then((response) => {
                        Object.assign(tagAssociationOne, response.body);
                        assert.strictEqual(
                            response.body.mac.toUpperCase(),
                            pairOne.mac,
                            `Wrong tag association 1 "mac" value`
                        );
                        assert.strictEqual(
                            response.body.trackable_uid,
                            pairOne.uid,
                            `Wrong tag association 1 "trackable_uid" value`
                        );
                        resolve(response.body);
                    })
                    .catch((err) => reject(err));
            });

            const promise2 = new Promise((resolve, reject) => {
                post('/api/v3/tag_assignments', bodyTwo)
                    .expect(200)
                    .then((response) => {
                        Object.assign(tagAssociationTwo, response.body);
                        assert.strictEqual(
                            response.body.mac.toUpperCase(),
                            pairTwo.mac,
                            `Wrong tag association 2 "mac" value`
                        );
                        assert.strictEqual(
                            response.body.trackable_uid,
                            pairTwo.uid,
                            `Wrong tag association 2 "trackable_uid" value`
                        );
                        resolve(response.body);
                    })
                    .catch((err) => reject(err));
            });
            const TEST_ROUNDS = 101;

            // for (let i of [...Array(TEST_ROUNDS).keys()]) {
            for (let i = 0; i < TEST_ROUNDS; i = i + 1) {
                const isOdd = !!((i + 1) % 2);
                pairOne = isOdd ? oddPair1 : evenPair1;
                pairTwo = isOdd ? oddPair2 : evenPair2;
                bodyOne = isOdd
                    ? {
                          mac: oddPair1.mac,
                          trackable_uid: oddPair1.uid,
                      }
                    : {
                          mac: evenPair1.mac,
                          trackable_uid: evenPair1.uid,
                      };
                bodyTwo = isOdd
                    ? {
                          mac: oddPair2.mac,
                          trackable_uid: oddPair2.uid,
                      }
                    : {
                          mac: evenPair2.mac,
                          trackable_uid: evenPair2.uid,
                      };

                await Promise.all([promise1, promise2]);
                // console.log(responses);

                const delPromiseOne = put(`/api/v3/tag_assignments`, { mac: bodyOne.mac });

                const delPromiseTwo = put(`/api/v3/tag_assignments`, { mac: bodyTwo.mac });
                await Promise.all([delPromiseOne, delPromiseTwo]);
                // console.log(`round: ${i}`);
            }
        });
    });
});
