import assert from 'assert';
import { UID } from '../../lib/utils/index.mjs';
import { get, post, randomInt } from '../utils/index.mjs';
import { main } from '../../appSetup.mjs';

let initialized;
export const Server = () => {
    const serverStart = () => {
        return main();
    }
    
    return {
        init: () => {
            if (!initialized) {
                initialized = serverStart();
            }
            return initialized;
        }
    }
}

/**
 * @function
 * @param {String} mac - MAC address of the hardware device
 * @return {Promise<Object>} - created device
 */
export async function createDevice(mac) {
    const body = { mac };
    const response = await post('/api/v3/hardware_devices', body);
    return response.body;
}

/**
 * @function
 * @param {String} mac - MAC address of the hardware device
 * @return {Promise<Object>} - created node
 */
export async function createNode(mac) {
    const body = { mac };
    const response = await post('/api/v3/nodes', body);
    return response.body;
}

/**
 * @function
 * @param {String} mac - MAC address of the hardware device
 * @return {Promise<Object>} - created node
 */
export async function createTag(mac) {
    const body = { mac };
    const response = await post('/api/v3/tags', body);
    return response.body;
}

/**
 * @function
 * @param {String} name - location name
 * @return {Promise<Object>} - created location
 */
export async function createLocation(name) {
    const body = { name };
    const response = await post('/api/v3/locations', body);
    return response.body;
}

/**
 * @function
 * @param {String} name - sublocation name
 * @param {Object} location - parent location
 * @return {Promise<Object>} - created sublocation
 */
export async function createSublocation(name, location) {
    const body = { name, location_uid: location.uid };
    const response = await post('/api/v3/sublocations', body);
    return response.body;
}

/**
 * @function
 * @param {String} name - anchor name
 * @param {Object} sublocation - parent sublocation
 * @return {Promise<Object>} - created anchor
 */
export async function createAnchor(name, sublocation) {
    const body = {
        name,
        bridge: false,
        position: { x: randomInt(), y: randomInt(), z: randomInt(), sublocation_uid: sublocation.uid },
    };
    const response = await post('/api/v3/anchors', body);
    return response.body;
}

/**
 * @function
 * @param {String} name - trackable name
 * @param {Object|null} position - trackable position
 * @return {Promise<Object>} - created trackable
 */
export async function createTrackable(name, position = null) {
    const body = {
        name,
        position,
        position_reporting: {
            interval: 0.25,
            auto: false,
        },
    };
    const response = await post('/api/v3/trackables', body);
    return response.body;
}

/**
 * @function
 * @param {String} locationUid - location uid
 * @param {String} name - schedule name
 * @return {Promise<Object>} - created trackable
 */
export async function createLsbSchedule(locationUid, name) {
    const body = {
        location_uid: locationUid,
        name,
        content: [
            {
                source: UID(),
                destination: UID(),
                slot: 0,
                section: 1,
                dimension: '0D',
                boundary: true,
                margin: 250,
            },
        ],
    };
    const response = await post('/api/v3/network/lsb_schedules', body);
    return response.body;
}

/**
 * @function
 * @param {Object} anchor - anchor object
 * @param {String} anchor.uid - unique identifier of the anchor
 * @param {Object} node - node object
 * @param {String} node.mac - MAC address of the hardware device
 * @return {Promise<Object>} - created node association
 */
export async function createNodeAssociation(anchor, node) {
    const body = { mac: node.mac, anchor_uid: anchor.uid };
    const response = await post('/api/v3/node_assignments', body);
    return response.body;
}

/**
 * @function
 * @param {Object} trackable - trackable object
 * @param {String} trackable.uid - unique identifier of the trackable
 * @param {Object} tag - tag object
 * @param {String} tag.mac - MAC address of the hardware device
 * @return {Promise<Object>} - created tag association
 */
export async function createTagAssociation(trackable, tag) {
    const body = { mac: tag.mac, trackable_uid: trackable.uid };
    const response = await post('/api/v3/tag_assignments', body);
    return response.body;
}

/**
 * @function
 * @param {String} route - route name
 * @param {Array<Object>} items - the list of entities
 * @param {String} entity - entity name
 * @return {Function}
 */
export function checkEntities(route, items, entity = undefined) {
    return function (done) {
        entity = entity || route;
        get(`/api/v3/${route}`)
            .expect(200)
            .then((response) => {
                assert.deepStrictEqual(
                    response.body,
                    { [`${entity}`]: items },
                    `Not equal to: { ${entity}: ${items} }`
                );
                done();
            })
            .catch((err) => done(err));
    };
}
