import assert from 'assert';
import { UID } from '../../lib/utils/index.mjs';
import { ERROR_CODE, ZMQ_ACTIONS, ZMQ_CHANNELS, ZMQ_OBJECTS } from '../../lib/system/constants.mjs';
import { isISOTimeFormat, isUpperCaseString, ZmqObserver, get, post, put, del } from '../utils/index.mjs';
import {
    createTrackable,
    createAnchor,
    createNode,
    createTag,
    checkEntities,
    createLocation,
    createSublocation,
    Server,
} from './helpers.mjs';

describe('Node Associations Management API', function () {
    const nodeAssociationOne = {};
    const nodeAssociationTwo = {};
    const nodeAssociationThree = {};
    const anchorOne = {};
    const anchorTwo = {};
    const anchorThree = {};
    const nodeOne = {};
    const nodeTwo = {};
    const nodeThree = {};
    const tag = {};
    const trackable = {};
    const location = {};
    const sublocation = {};
    const anchorCount = 3;

    before(async function () {
        await Server().init();
        Object.assign(location, await createLocation(`Location ${UID()}`));
        Object.assign(sublocation, await createSublocation(`Sublocation ${UID()}`, location));
        Object.assign(trackable, await createTrackable(`Trackable ${UID()}`));
        Object.assign(anchorOne, await createAnchor(`Anchor ${UID()}`, sublocation));
        Object.assign(anchorTwo, await createAnchor(`Anchor ${UID()}`, sublocation));
        Object.assign(anchorThree, await createAnchor(`Anchor ${UID()}`, sublocation));
        Object.assign(nodeOne, await createNode('04956EA5010c'));
        Object.assign(nodeTwo, await createNode('04956EA5010b'));
        Object.assign(nodeThree, await createNode('04956EA5010d'));
        Object.assign(tag, await createTag('04956EA5010a'));
    });

    after(async function () {
        await del(`/api/v3/locations/${location.uid}`); // cascade remove of sublocations/anchors
        await del(`/api/v3/trackables/${trackable.uid}`);
        await del(`/api/v3/nodes/${nodeOne.mac}`);
        await del(`/api/v3/nodes/${nodeTwo.mac}`);
        await del(`/api/v3/nodes/${nodeThree.mac}`);
        await del(`/api/v3/tags/${tag.mac}`);
    });

    describe('Anchor before node association creation', function () {
        it('should return list of anchors with node_mac equal to null', function (done) {
            get(`/api/v3/anchors`)
                .expect(200)
                .then((response) => {
                    const { anchors } = response.body;
                    assert(anchors.length === anchorCount, `Wrong anchors number`);
                    for (const anchor of anchors) {
                        assert(anchor.node_mac === null, 'anchor should have mac link equal null');
                    }
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('Get Empty Node Associations List', function () {
        it('should return empty list of node associations', checkEntities('node_assignments', []));
    });

    describe('Create Node Association', function () {
        describe('Negative cases 1', function () {
            it('should fail create node association when body without anchor_uid', function (done) {
                const body = {
                    mac: nodeOne.mac,
                };

                post('/api/v3/node_assignments', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"anchor_uid":"REQUIRED"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create node association when body with empty anchor_uid', function (done) {
                const body = {
                    mac: nodeOne.mac,
                    anchor_uid: '',
                };

                post('/api/v3/node_assignments', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"anchor_uid":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create node association with trackable', function (done) {
                const body = {
                    mac: nodeOne.mac,
                    anchor_uid: trackable.uid,
                };

                post('/api/v3/node_assignments', body)
                    .expect(404)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            `Anchor with uid ${trackable.uid} not found`,
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create node association when body without mac', function (done) {
                const body = {
                    anchor_uid: anchorOne.uid,
                };

                post('/api/v3/node_assignments', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"mac":"REQUIRED"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create node association when body is empty mac', function (done) {
                const body = {
                    anchor_uid: anchorOne.uid,
                    mac: '',
                };

                post('/api/v3/node_assignments', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"mac":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create tag association when body is wrong mac, WRONG_MAC_FORMAT', function (done) {
                const body = {
                    anchor_uid: anchorOne.uid,
                    mac: 'MAC12345IT22',
                };

                put('/api/v3/node_assignments', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"mac":"WRONG_MAC_FORMAT"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create node association with non existing anchor', function (done) {
                const body = {
                    mac: nodeOne.mac,
                    anchor_uid: UID(),
                };

                post('/api/v3/node_assignments', body)
                    .expect(404)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            `Anchor with uid ${body.anchor_uid} not found`,
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create node association with non existing node', function (done) {
                const body = {
                    mac: '00000000001a',
                    anchor_uid: anchorOne.uid,
                };

                post('/api/v3/node_assignments', body)
                    .expect(404)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            `Node with mac ${body.mac.toUpperCase()} not found`,
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create node association with tag', function (done) {
                const body = {
                    mac: tag.mac,
                    anchor_uid: anchorOne.uid,
                };

                post('/api/v3/node_assignments', body)
                    .expect(404)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, `Node with mac ${body.mac} not found`, `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
        });
        describe('Create first', function () {
            it('should create first node association', function (done) {
                const body = {
                    mac: nodeOne.mac.toLowerCase(),
                    anchor_uid: anchorOne.uid,
                };

                post('/api/v3/node_assignments', body)
                    .expect(200)
                    .then((response) => {
                        Object.assign(nodeAssociationOne, response.body);
                        assert.strictEqual(
                            response.body.mac.toUpperCase(),
                            body.mac.toUpperCase(),
                            `Wrong node association "mac" value`
                        );
                        assert.strictEqual(
                            response.body.anchor_uid,
                            body.anchor_uid,
                            `Wrong node association "anchor_uid" value`
                        );
                        assert(isUpperCaseString(response.body.mac), `Response MAC address should be in uppercase`);
                        assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                        done();
                    })
                    .catch((err) => done(err));
            });
        });
        describe('Negative cases 2', function () {
            it('should fail create node association with short node MAC address', function (done) {
                const body = {
                    mac: '0000001a',
                    anchor_uid: anchorOne.uid,
                };

                post('/api/v3/node_assignments', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"mac":"WRONG_MAC_FORMAT"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create node association with too long node MAC address', function (done) {
                const body = {
                    mac: '00000000000000001a',
                    anchor_uid: anchorOne.uid,
                };

                post('/api/v3/node_assignments', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"mac":"WRONG_MAC_FORMAT"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create node association with node in association', function (done) {
                const body = {
                    mac: nodeOne.mac,
                    anchor_uid: anchorTwo.uid,
                };

                post('/api/v3/node_assignments', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            `Assignment with node ${body.mac} already exist`,
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create node association with anchor in association', function (done) {
                const body = {
                    mac: nodeTwo.mac,
                    anchor_uid: anchorOne.uid,
                };

                post('/api/v3/node_assignments', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            `Assignment with anchor ${body.anchor_uid} already exist`,
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
        });
        describe('Create next', function () {
            it('should create second node association', function (done) {
                const body = {
                    mac: nodeTwo.mac,
                    anchor_uid: anchorTwo.uid,
                };

                post('/api/v3/node_assignments', body)
                    .expect(200)
                    .then((response) => {
                        Object.assign(nodeAssociationTwo, response.body);
                        assert.strictEqual(response.body.mac, body.mac, `Wrong node association "mac" value`);
                        assert.strictEqual(
                            response.body.anchor_uid,
                            body.anchor_uid,
                            `Wrong node association "anchor_uid" value`
                        );
                        assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                        assert(isUpperCaseString(response.body.mac), `Response MAC address should be in uppercase`);
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should create third node association', function (done) {
                const body = {
                    mac: nodeThree.mac,
                    anchor_uid: anchorThree.uid,
                };

                post('/api/v3/node_assignments', body)
                    .expect(200)
                    .then((response) => {
                        Object.assign(nodeAssociationThree, response.body);
                        assert.strictEqual(response.body.mac, body.mac, `Wrong node association "mac" value`);
                        assert.strictEqual(
                            response.body.anchor_uid,
                            body.anchor_uid,
                            `Wrong node association "anchor_uid" value`
                        );
                        assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                        assert(isUpperCaseString(response.body.mac), `Response MAC address should be in uppercase`);
                        done();
                    })
                    .catch((err) => done(err));
            });
        });
    });

    describe('Anchor after node association creation', function () {
        it('should return list of anchors with node_mac equal to null', function (done) {
            get(`/api/v3/anchors`)
                .expect(200)
                .then((response) => {
                    const { anchors } = response.body;
                    const nodeAssociations = [nodeAssociationOne, nodeAssociationTwo, nodeAssociationThree];
                    assert(anchors.length === anchorCount, `Wrong anchors number`);
                    for (const anchor of anchors) {
                        const nodeAssociation = nodeAssociations.find((item) => item.anchor_uid === anchor.uid);
                        assert(anchor.node_mac === nodeAssociation.mac, 'anchor should have mac link');
                    }
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('Close Node Association Management', function () {
        describe('Negative cases', function () {
            it('should return ValidationError error on empty body', function (done) {
                const body = {};
                put(`/api/v3/node_assignments`, body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            `{"mac":"REQUIRED","anchor_uid":"REQUIRED"}`,
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fails close first node association with empty mac', function (done) {
                const body = {
                    mac: '',
                };

                put('/api/v3/node_assignments', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"mac":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fails close first node association with empty anchor_uid', function (done) {
                const body = {
                    anchor_uid: '',
                };

                put('/api/v3/node_assignments', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"anchor_uid":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fails close first node association with empty mac and empty anchor_uid', function (done) {
                const body = {
                    mac: '',
                    anchor_uid: '',
                };

                put('/api/v3/node_assignments', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            '{"mac":"CANNOT_BE_EMPTY","anchor_uid":"CANNOT_BE_EMPTY"}',
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create node association when body is empty mac, CANNOT_BE_EMPTY', function (done) {
                const body = {
                    anchor_uid: anchorOne.uid,
                    mac: '',
                };

                put('/api/v3/node_assignments', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"mac":"CANNOT_BE_EMPTY"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fail create node association when body is wrong mac, WRONG_MAC_FORMAT', function (done) {
                const body = {
                    anchor_uid: anchorOne.uid,
                    mac: 'MAC12345IT22',
                };

                put('/api/v3/node_assignments', body)
                    .expect(422)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(message, '{"mac":"WRONG_MAC_FORMAT"}', `Wrong error message`);
                        assert.deepStrictEqual(code, ERROR_CODE.VALIDATION_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should return NotFound error non existing mac association', function (done) {
                const body = {
                    mac: '00000000000a',
                };
                put(`/api/v3/node_assignments`, body)
                    .expect(404)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            `Assignment with node ${body.mac.toUpperCase()} not found`,
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fails close first node association when correct mac wrong anchor_uid ', function (done) {
                const body = {
                    mac: nodeOne.mac.toLowerCase(),
                    anchor_uid: 'sdasd',
                };

                put('/api/v3/node_assignments', body)
                    .expect(404)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            `Assignment with node ${body.mac.toUpperCase()} and anchor ${body.anchor_uid} not found`,
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should fails close first node association when correct anchor_uid wrong mac ', function (done) {
                const body = {
                    mac: '000ee000002a',
                    anchor_uid: anchorOne.uid,
                };

                put('/api/v3/node_assignments', body)
                    .expect(404)
                    .then((response) => {
                        const { message, code } = response.body;
                        assert.deepStrictEqual(
                            message,
                            `Assignment with node ${body.mac.toUpperCase()} and anchor ${body.anchor_uid} not found`,
                            `Wrong error message`
                        );
                        assert.deepStrictEqual(code, ERROR_CODE.NOT_FOUND_ERROR, 'Wrong error code');
                        done();
                    })
                    .catch((err) => done(err));
            });
        });
        describe('Positive cases', function () {
            it('should close first node association', function (done) {
                const body = {
                    mac: nodeAssociationOne.mac.toLowerCase(),
                };
                put(`/api/v3/node_assignments`, body)
                    .expect(200)
                    .then((response) => {
                        assert.strictEqual(
                            response.body.mac.toUpperCase(),
                            body.mac.toUpperCase(),
                            `Wrong node association "mac" value`
                        );
                        assert.strictEqual(
                            response.body.anchor_uid,
                            anchorOne.uid,
                            `Wrong node association "anchor_uid" value`
                        );
                        assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                        assert(isUpperCaseString(response.body.mac), `Response MAC address should be in uppercase`);
                        done();
                    })
                    .catch((err) => done(err));
            });
            it('should re-create node association', function (done) {
                const body = {
                    mac: nodeOne.mac.toLowerCase(),
                    anchor_uid: anchorOne.uid,
                };
                post(`/api/v3/node_assignments`, body)
                    .expect(200)
                    .then((response) => {
                        Object.assign(nodeAssociationOne, response.body);
                        assert.strictEqual(
                            response.body.mac.toUpperCase(),
                            body.mac.toUpperCase(),
                            `Wrong node association "mac" value`
                        );
                        assert.strictEqual(
                            response.body.anchor_uid,
                            body.anchor_uid,
                            `Wrong node association "anchor_uid" value`
                        );
                        assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                        assert(isUpperCaseString(response.body.mac), `Response MAC address should be in uppercase`);
                        done();
                    })
                    .catch((err) => done(err));
            });
        });
    });

    describe('Clear Node Association', function () {
        it('should return list of node associations', function (done) {
            get('/api/v3/node_assignments')
                .expect(200)
                .then((response) => {
                    const resAssociationOne = response.body.node_assignments.find(
                        (assoc) => assoc.anchor_uid === nodeAssociationOne.anchor_uid
                    );
                    const resAssociationTwo = response.body.node_assignments.find(
                        (assoc) => assoc.anchor_uid === nodeAssociationTwo.anchor_uid
                    );
                    assert.strictEqual(
                        response.body.node_assignments.length,
                        3,
                        'Associations list should contain 3 items'
                    );
                    assert.deepStrictEqual(resAssociationOne, nodeAssociationOne, 'Associations should be equal');
                    assert.deepStrictEqual(resAssociationTwo, nodeAssociationTwo, 'Associations should be equal');
                    done();
                })
                .catch((err) => done(err));
        });
        it('should close node association one by anchor_uid', function (done) {
            const body = { anchor_uid: anchorOne.uid };
            put(`/api/v3/node_assignments`, body)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(
                        response.body.anchor_uid.toUpperCase(),
                        body.anchor_uid.toUpperCase(),
                        `Wrong node association "anchor_uid" value`
                    );
                    assert.strictEqual(
                        response.body.anchor_uid,
                        nodeAssociationOne.anchor_uid,
                        `Wrong node association "anchor_uid" value`
                    );
                    assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should close node association two by mac', function (done) {
            const body = { mac: nodeAssociationTwo.mac };
            put(`/api/v3/node_assignments`, body)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(
                        response.body.mac.toUpperCase(),
                        body.mac.toUpperCase(),
                        `Wrong node association "mac" value`
                    );
                    assert.strictEqual(
                        response.body.anchor_uid,
                        nodeAssociationTwo.anchor_uid,
                        `Wrong node association "anchor_uid" value`
                    );
                    assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should close node association three by anchor and mac', function (done) {
            const body = {
                anchor_uid: nodeAssociationThree.anchor_uid,
                mac: nodeAssociationThree.mac,
            };
            put(`/api/v3/node_assignments`, body)
                .expect(200)
                .then((response) => {
                    assert.strictEqual(
                        response.body.mac.toUpperCase(),
                        body.mac.toUpperCase(),
                        `Wrong node association "mac" value`
                    );
                    assert.strictEqual(
                        response.body.anchor_uid,
                        nodeAssociationThree.anchor_uid,
                        `Wrong node association "anchor_uid" value`
                    );
                    assert(isISOTimeFormat(response.body.created_at), `Wrong anchor "created_at" format`);
                    done();
                })
                .catch((err) => done(err));
        });
        it('should return empty list of node associations after cleanup', checkEntities('node_assignments', []));
    });

    describe('Anchor after node association removing', function () {
        it('should return list of anchors with node_mac equal to null', function (done) {
            get(`/api/v3/anchors`)
                .expect(200)
                .then((response) => {
                    const { anchors } = response.body;
                    assert(anchors.length === anchorCount, `Wrong anchors number`);
                    for (const anchor of anchors) {
                        assert(anchor.node_mac === null, 'anchor should have mac link equal null');
                    }
                    done();
                })
                .catch((err) => done(err));
        });
    });

    describe('ZMQ Messaging for Node Assignments CRUD', function () {
        const zmqObserver = new ZmqObserver();
        const nodeAssociationZmq = {};

        before(function () {
            zmqObserver.init();
        });
        after(function () {
            zmqObserver.destroy();
        });

        it('should send zmq created node assignment message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.CREATED, object: ZMQ_OBJECTS.NODE_ASSIGNMENT };
            const body = {
                mac: nodeOne.mac.toLowerCase(),
                anchor_uid: anchorOne.uid,
            };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert(zmqMsg.data.mac === body.mac.toUpperCase(), 'ZMQ message has wrong assignment mac address');
                assert(zmqMsg.data.anchor_uid === body.anchor_uid, 'ZMQ message has wrong anchor_uid prop');
                assert(isISOTimeFormat(zmqMsg.data.created_at), 'Wrong ZMQ message created_at value');
                done();
            });
            subscription
                .then(() => post(`/api/v3/node_assignments`, body))
                .then((response) => {
                    Object.assign(nodeAssociationZmq, response.body);
                });
        });

        it('should send zmq deleted node assignment message', function (done) {
            const zmqSubscription = { action: ZMQ_ACTIONS.DELETED, object: ZMQ_OBJECTS.NODE_ASSIGNMENT };

            const subscription = zmqObserver.subscribe(zmqSubscription, function (zmqMsg, channel) {
                assert(isISOTimeFormat(zmqMsg.ts), `Wrong ZMQ message "ts" format`);
                assert(channel === ZMQ_CHANNELS.EVENT, 'Wrong ZMQ message channel');
                assert(
                    zmqMsg.data.mac === nodeAssociationZmq.mac.toUpperCase(),
                    'ZMQ message has wrong assignment mac address'
                );
                assert(
                    zmqMsg.data.anchor_uid === nodeAssociationZmq.anchor_uid,
                    'ZMQ message has wrong anchor_uid prop'
                );
                assert(isISOTimeFormat(zmqMsg.data.created_at), 'Wrong ZMQ message created_at value');
                done();
            });
            subscription.then(() => put(`/api/v3/node_assignments`, nodeAssociationZmq));
        });
    });
});
