import makeRequestHandler from '../../../utils/makeRequestHandler.mjs';
import { AnchorGet, AnchorDelete, AnchorSave, AnchorUpdate } from '../../../../usecases/index.mjs';

export default {
    /**
     * @swagger
     * /api/v3/anchors:
     *   get:
     *     description: Retrieve anchors list
     *     tags:
     *       - Anchors API
     *     responses:
     *       '200':
     *         description: anchors
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/Anchors'
     *              example:
     *                  anchors:
     *                      -
     *                          uid: eJEG2Q2kRnSHJaBdHxQ3Pw
     *                          name: Anchor 22e
     *                          bridge: false
     *                          created_at: 2021-04-21T10:00:00.000Z
     *                          updated_at: 2021-04-21T10:00:00.000Z
     *                          node_mac: E4956EA501FC
     *                          position:
     *                              x: 1000
     *                              y: 2000
     *                              z: 1000
     *                              sublocation_uid: z2kq0eIRQ92ZH87jb0BUvQ
     *                      -
     *                          uid: RJew2klzptrSdFwAFxj3Pw
     *                          name: Bridge 1
     *                          bridge: true
     *                          created_at: 2021-04-21T10:00:00.000Z
     *                          updated_at: 2021-04-21T10:00:00.000Z
     *                          node_mac: E4956EA501AB
     *                          position:
     *                              x: 2000
     *                              y: 3000
     *                              z: 5000
     *                              sublocation_uid: z2kq0eIRQ92ZH87jb0BUvQ
     */
    getAll: makeRequestHandler(
        AnchorGet,
        () => ({}),
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/anchors/{uid}:
     *   get:
     *     description: Retrieve single anchor
     *     tags:
     *       - Anchors API
     *     parameters:
     *      - in: path
     *        name: uid
     *        schema:
     *          type: string
     *          required: true
     *          example: eJEG2Q2kRnSHJaBdHxQ3Pw
     *          description: anchor unique identifier
     *     responses:
     *       '200':
     *         description: anchors
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/Anchors'
     *              example:
     *                  anchors:
     *                      -
     *                          uid: eJEG2Q2kRnSHJaBdHxQ3Pw
     *                          name: Anchor 22e
     *                          bridge: false
     *                          created_at: 2021-04-21T10:00:00.000Z
     *                          updated_at: 2021-04-21T10:00:00.000Z
     *                          node_mac: E4956EA501FC
     *                          position:
     *                              x: 1000
     *                              y: 2000
     *                              z: 1000
     *                              sublocation_uid: z2kq0eIRQ92ZH87jb0BUvQ
     *       '404':
     *         description: An anchor with the specified UID was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              example:
     *                  message: Anchor with uid ... not found
     *                  code: NOT_FOUND_ERROR
     */
    get: makeRequestHandler(
        AnchorGet,
        (req) => ({
            uid: req.params.uid,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/anchors:
     *   post:
     *     description: Create anchor
     *     tags:
     *       - Anchors API
     *     requestBody:
     *      content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/AnchorCreateBody'
     *              example:
     *                  name: Anchor 221
     *                  bridge: false
     *                  position:
     *                      x: 1000
     *                      y: 2000
     *                      z: 4000
     *                      sublocation_uid: z2kq0eIRQ92ZH87jb0BUvQ
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/Anchor'
     *              example:
     *                  uid: eJEG2Q2kRnSHJaBdHxQ3Pw
     *                  name: Anchor 221
     *                  bridge: false
     *                  created_at: 2021-04-21T10:00:00.000Z
     *                  updated_at: 2021-04-21T10:00:00.000Z
     *                  node_mac: null
     *                  position:
     *                      x: 1000
     *                      y: 2000
     *                      z: 4000
     *                      sublocation_uid: z2kq0eIRQ92ZH87jb0BUvQ
     *       '422':
     *         description: Invalid request parameters.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/ValidationError'
     *              examples:
     *                  Name not specify:
     *                      value:
     *                          message: '{"name":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Empty name:
     *                      value:
     *                          message: '{"name":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  Bridge not specify:
     *                      value:
     *                          message: '{"bridge":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Empty bridge:
     *                      value:
     *                          message: '{"bridge":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  Position not specify:
     *                      value:
     *                          message: '{"position":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.x:
     *                      value:
     *                          message: '{"position":{"x":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify but position.x should be integer:
     *                      value:
     *                          message: '{"position":{"x":"NOT_INTEGER"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.y:
     *                      value:
     *                          message: '{"position":{"y":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify but position.y should be integer:
     *                      value:
     *                          message: '{"position":{"y":"NOT_INTEGER"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.z:
     *                      value:
     *                          message: '{"position":{"z":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify but position.z should be integer:
     *                      value:
     *                          message: '{"position":{"z":"NOT_INTEGER"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.sublocation_uid:
     *                      value:
     *                          message: '{"position":{"sublocation_uid":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but empty position.sublocation_uid:
     *                      value:
     *                          message: '{"position":{"sublocation_uid":"CANNOT_BE_EMPTY"}}'
     *                          code: VALIDATION_ERROR
     *       '404':
     *         description: Sublocation for anchor position was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              example:
     *                  message: 'Sublocation with uid ... not found'
     *                  code: NOT_FOUND_ERROR
     */
    post: makeRequestHandler(
        AnchorSave,
        (req) => ({
            name: req.body.name,
            bridge: req.body.bridge,
            position: req.body.position,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/anchors/{uid}:
     *   put:
     *     description: Update anchor by uid
     *     tags:
     *       - Anchors API
     *     parameters:
     *      - in: path
     *        name: uid
     *        schema:
     *          type: string
     *          required: true
     *          example: eJEG2Q2kRnSHJaBdHxQ3Pw
     *          description: anchor unique identifier
     *     requestBody:
     *      content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/AnchorUpdateBody'
     *              example:
     *                  name: Anchor 222
     *                  bridge: true
     *                  position:
     *                      x: 1000
     *                      y: 2000
     *                      z: 4000
     *                      sublocation_uid: z2kq0eIRQ92ZH87jb0BUvQ
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/Anchor'
     *              example:
     *                  uid: eJEG2Q2kRnSHJaBdHxQ3Pw
     *                  name: Anchor 222
     *                  bridge: true
     *                  created_at: 2021-04-21T10:00:00.000Z
     *                  updated_at: 2021-04-21T10:00:00.000Z
     *                  node_mac: null
     *                  position:
     *                      x: 1000
     *                      y: 2000
     *                      z: 4000
     *                      sublocation_uid: z2kq0eIRQ92ZH87jb0BUvQ
     *       '404':
     *         description: An anchor with the specified UID was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              examples:
     *                  Non existed anchor:
     *                      value:
     *                          message: Anchor with uid ... not found
     *                          code: NOT_FOUND_ERROR
     *                  Non existed sublocation:
     *                      value:
     *                          message: 'Sublocation with uid ... not found'
     *                          code: NOT_FOUND_ERROR
     *       '422':
     *         description: Invalid request parameters.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/ValidationError'
     *              examples:
     *                  Empty name:
     *                      value:
     *                          message: '{"name":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  Empty bridge:
     *                      value:
     *                          message: '{"bridge":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.x:
     *                      value:
     *                          message: '{"position":{"x":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify but position.x should be integer:
     *                      value:
     *                          message: '{"position":{"x":"NOT_INTEGER"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.y:
     *                      value:
     *                          message: '{"position":{"y":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify but position.y should be integer:
     *                      value:
     *                          message: '{"position":{"y":"NOT_INTEGER"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.z:
     *                      value:
     *                          message: '{"position":{"z":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify but position.z should be integer:
     *                      value:
     *                          message: '{"position":{"z":"NOT_INTEGER"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but no position.sublocation_uid:
     *                      value:
     *                          message: '{"position":{"sublocation_uid":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Position specify, but empty position.sublocation_uid:
     *                      value:
     *                          message: '{"position":{"sublocation_uid":"CANNOT_BE_EMPTY"}}'
     *                          code: VALIDATION_ERROR
     */
    put: makeRequestHandler(
        AnchorUpdate,
        (req) => ({
            uid: req.params.uid,
            name: req.body.name,
            bridge: req.body.bridge,
            position: req.body.position,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/anchors/{uid}:
     *   delete:
     *     description: Remove anchor by uid
     *     tags:
     *       - Anchors API
     *     parameters:
     *      - in: path
     *        name: uid
     *        schema:
     *          type: string
     *          required: true
     *          example: eJEG2Q2kRnSHJaBdHxQ3Pw
     *          description: anchor unique identifier
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  type: object
     *                  properties:
     *                      uid:
     *                          type: string
     *              example:
     *                  uid: eJEG2Q2kRnSHJaBdHxQ3Pw
     *       '404':
     *         description: An anchor with the specified UID was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              example:
     *                  message: Anchor with uid ... not found
     *                  code: NOT_FOUND_ERROR
     */
    del: makeRequestHandler(
        AnchorDelete,
        (req) => ({
            uid: req.params && req.params.uid,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
};
