import makeRequestHandler from '../../../utils/makeRequestHandler.mjs';
import {
    NetworkLsbScheduleGet,
    NetworkLsbScheduleDelete,
    NetworkLsbScheduleSave,
    NetworkLsbScheduleUpdate,
} from '../../../../usecases/index.mjs';

export default {
    /**
     * @swagger
     * /api/v3/network/lsb_schedules:
     *   get:
     *     description: Network LSB(location service broadcast) schedule
     *     tags:
     *       - Network LSB schedule API
     *     parameters:
     *      - in: query
     *        name: location_uid
     *        schema:
     *          type: string
     *          required: false
     *          example: kYuzyFjYQl65g0MqNvhaYA
     *          description: location unique identifier
     *     responses:
     *       '200':
     *         description: network schedule
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/LsbSchedules'
     *              example:
     *                  lsb_schedules:
     *                      -
     *                          uid: dzObCb7wT3ObUNN3tOhMxQ
     *                          location_uid: _2E2DIF5SAemIHbnIhLbGg
     *                          created_at: 2021-04-21T10:00:00.000Z
     *                          updated_at: 2021-04-21T10:00:00.000Z
     *                          name: Schedule #1
     *                          content:
     *                              -
     *                                  source: amk6KdGTSGGNSqROibEdrQ
     *                                  destination: eexWLBmmRk_Msqgxbinqdw
     *                                  slot: 0
     *                                  section: 0
     *                                  dimension: 2D
     *                                  boundary: false
     *                                  margin: 0
     */
    getAll: makeRequestHandler(
        NetworkLsbScheduleGet,
        (req) => ({
            location_uid: req.query.location_uid,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/network/lsb_schedules/{uid}:
     *   get:
     *     description: Retrieve network schedule item by uid
     *     tags:
     *       - Network LSB schedule API
     *     parameters:
     *      - in: path
     *        name: uid
     *        schema:
     *          type: string
     *          required: true
     *          example: kYuzyFjYQl65g0MqNvhaYA
     *          description: network LSB schedule unique identifier
     *     responses:
     *       '200':
     *         description: network schedule
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/LsbSchedules'
     *              example:
     *                  lsb_schedules:
     *                      -
     *                          uid: dzObCb7wT3ObUNN3tOhMxQ
     *                          location_uid: _2E2DIF5SAemIHbnIhLbGg
     *                          created_at: 2021-04-21T10:00:00.000Z
     *                          updated_at: 2021-04-21T10:00:00.000Z
     *                          name: Schedule #1
     *                          content:
     *                              -
     *                                  source: amk6KdGTSGGNSqROibEdrQ
     *                                  destination: eexWLBmmRk_Msqgxbinqdw
     *                                  slot: 0
     *                                  section: 0
     *                                  dimension: 2D
     *                                  boundary: false
     *                                  margin: 0
     *       '404':
     *          description: Network schedule with the specified UID was not found.
     *          content:
     *              application/json:
     *                  schema:
     *                      $ref: '#/components/schemas/NotFoundError'
     *                  examples:
     *                      Non existed network schedule:
     *                          value:
     *                              message: Network schedule with uid ... not found
     *                              code: NOT_FOUND_ERROR
     */
    get: makeRequestHandler(
        NetworkLsbScheduleGet,
        (req) => ({
            uid: req.params.uid,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/network/lsb_schedules:
     *   post:
     *     description: Create network LSB schedule
     *     tags:
     *       - Network LSB schedule API
     *     requestBody:
     *      content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/LsbScheduleCreateBody'
     *              example:
     *                  location_uid: _2E2DIF5SAemIHbnIhLbGg
     *                  name: Schedule #1
     *                  content:
     *                      -
     *                          source: amk6KdGTSGGNSqROibEdrQ
     *                          destination: eexWLBmmRk_Msqgxbinqdw
     *                          slot: 0
     *                          section: 0
     *                          dimension: 2D
     *                          boundary: true
     *                          margin: 0
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/LsbSchedule'
     *              example:
     *                  uid: dzObCb7wT3ObUNN3tOhMxQ
     *                  location_uid: _2E2DIF5SAemIHbnIhLbGg
     *                  name: Schedule #1
     *                  created_at: 2021-04-21T10:00:00.000Z
     *                  updated_at: 2021-04-21T10:00:00.000Z
     *                  content:
     *                      -
     *                          source: amk6KdGTSGGNSqROibEdrQ
     *                          destination: eexWLBmmRk_Msqgxbinqdw
     *                          slot: 0
     *                          section: 0
     *                          dimension: 2D
     *                          boundary: true
     *                          margin: 0
     *       '404':
     *          description: Location with the specified UID was not found.
     *          content:
     *              application/json:
     *                  schema:
     *                      $ref: '#/components/schemas/NotFoundError'
     *                  examples:
     *                      Non existed location:
     *                          value:
     *                              message: Location with uid ... not found
     *                              code: NOT_FOUND_ERROR
     *                      Non existed schedule:
     *                          value:
     *                              message: LSB schedule with uid ... not found
     *                              code: NOT_FOUND_ERROR
     *       '422':
     *         description: Invalid request parameters.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/ValidationError'
     *              examples:
     *                  No location_uid specify:
     *                      value:
     *                          message: '{"location_uid":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Empty location_uid:
     *                      value:
     *                          message: '{"location_uid":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  No name specify:
     *                      value:
     *                          message: '{"name":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Empty name:
     *                      value:
     *                          message: '{"name":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  No content specify:
     *                      value:
     *                          message: '{"content":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Empty content:
     *                      value:
     *                          message: '{"content":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  No content.source specify:
     *                      value:
     *                          message: '{"content":[{"source":"REQUIRED"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.destination specify:
     *                      value:
     *                          message: '{"content":[{"destination":"REQUIRED"}]}'
     *                          code: VALIDATION_ERROR
     *                  Empty content.destination:
     *                      value:
     *                          message: '{"content":[{"destination":"CANNOT_BE_EMPTY"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.section specify:
     *                      value:
     *                          message: '{"content":[{"section":"REQUIRED"}]}'
     *                          code: VALIDATION_ERROR
     *                  Wrong content.section value, allowed [1, 0]:
     *                      value:
     *                          message: '{"content":[{"section":"NOT_ALLOWED_VALUE"}]}'
     *                          code: VALIDATION_ERROR
     *                  Content.section should be integer:
     *                      value:
     *                          message: '{"content":[{"section":"NOT_INTEGER"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.boundary specify:
     *                      value:
     *                          message: '{"content":[{"boundary":"REQUIRED"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.slot specify:
     *                      value:
     *                          message: '{"content":[{"slot":"REQUIRED"}]}'
     *                          code: VALIDATION_ERROR
     *                  Content.slot is too low:
     *                      value:
     *                          message: '{"content":[{"slot":"TOO_LOW"}]}'
     *                          code: VALIDATION_ERROR
     *                  Content.slot should be integer:
     *                      value:
     *                          message: '{"content":[{"slot":"NOT_INTEGER"}]}'
     *                          code: VALIDATION_ERROR
     *                  Content.slot is to high:
     *                      value:
     *                          message: '{"content":[{"slot":"TOO_HIGH"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.margin specify:
     *                      value:
     *                          message: '{"content":[{"margin":"REQUIRED"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.margin is too low:
     *                      value:
     *                          message: '{"content":[{"margin":"TOO_LOW"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.margin is to high:
     *                      value:
     *                          message: '{"content":[{"margin":"TOO_HIGH"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.margin not multiple to 250:
     *                      value:
     *                          message: '{"content":[{"margin":"NOT_MULTIPLE_TO 250"}]}'
     *                          code: VALIDATION_ERROR
     *                  Content.margin should be integer:
     *                      value:
     *                          message: '{"content":[{"margin":"NOT_INTEGER"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.dimension specify or empty:
     *                      value:
     *                          message: '{"content":[{"dimension":"REQUIRED"}]}'
     *                          code: VALIDATION_ERROR
     *                  No allow value content.dimension:
     *                      value:
     *                          message: '{"content":[{"dimension":"NOT_ALLOWED_VALUE"}]}'
     *                          code: VALIDATION_ERROR
     */
    post: makeRequestHandler(
        NetworkLsbScheduleSave,
        (req) => ({
            name: req.body.name,
            location_uid: req.body.location_uid,
            content: req.body.content,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/network/lsb_schedules/{uid}:
     *   put:
     *     description: Update network LSB schedule
     *     tags:
     *       - Network LSB schedule API
     *     parameters:
     *      - in: path
     *        name: uid
     *        schema:
     *          type: string
     *          required: true
     *          example: kYuzyFjYQl65g0MqNvhaYA
     *          description: network LSB schedule unique identifier
     *     requestBody:
     *      content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/LsbScheduleUpdateBody'
     *              example:
     *                  name: Schedule #2
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/LsbSchedule'
     *              example:
     *                  uid: dzObCb7wT3ObUNN3tOhMxQ
     *                  location_uid: _2E2DIF5SAemIHbnIhLbGg
     *                  name: Schedule #2
     *                  created_at: 2021-04-21T10:00:00.000Z
     *                  updated_at: 2021-04-21T10:00:00.000Z
     *                  content:
     *                      -
     *                          source: amk6KdGTSGGNSqROibEdrQ
     *                          destination: eexWLBmmRk_Msqgxbinqdw
     *                          slot: 0
     *                          section: 0
     *                          dimension: 2D
     *                          boundary: true
     *                          margin: 0
     *       '404':
     *          description: Network schedule with the specified UID was not found.
     *          content:
     *              application/json:
     *                  schema:
     *                      $ref: '#/components/schemas/NotFoundError'
     *                  examples:
     *                      Non existed network schedule:
     *                          value:
     *                              message: Network schedule with uid ... not found
     *                              code: NOT_FOUND_ERROR
     *       '422':
     *         description: Invalid request parameters.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/ValidationError'
     *              examples:
     *                  Empty name:
     *                      value:
     *                          message: '{"name":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  No content specify:
     *                      value:
     *                          message: '{"content":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Empty content:
     *                      value:
     *                          message: '{"content":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  No content.source specify:
     *                      value:
     *                          message: '{"content":[{"source":"REQUIRED"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.destination specify:
     *                      value:
     *                          message: '{"content":[{"destination":"REQUIRED"}]}'
     *                          code: VALIDATION_ERROR
     *                  Empty content.destination:
     *                      value:
     *                          message: '{"content":[{"destination":"CANNOT_BE_EMPTY"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.section specify:
     *                      value:
     *                          message: '{"content":[{"section":"REQUIRED"}]}'
     *                          code: VALIDATION_ERROR
     *                  Wrong content.section value, allowed [1, 0]:
     *                      value:
     *                          message: '{"content":[{"section":"NOT_ALLOWED_VALUE"}]}'
     *                          code: VALIDATION_ERROR
     *                  Content.section should be integer:
     *                      value:
     *                          message: '{"content":[{"section":"NOT_INTEGER"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.boundary specify:
     *                      value:
     *                          message: '{"content":[{"boundary":"REQUIRED"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.slot specify:
     *                      value:
     *                          message: '{"content":[{"slot":"REQUIRED"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.slot is too low:
     *                      value:
     *                          message: '{"content":[{"slot":"TOO_LOW"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.slot is to high:
     *                      value:
     *                          message: '{"content":[{"slot":"TOO_HIGH"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.slot is not integer:
     *                      value:
     *                          message: '{"content":[{"slot":"NOT_INTEGER"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.margin specify:
     *                      value:
     *                          message: '{"content":[{"margin":"REQUIRED"}]}'
     *                          code: VALIDATION_ERROR
     *                  Content.margin is too low:
     *                      value:
     *                          message: '{"content":[{"margin":"TOO_LOW"}]}'
     *                          code: VALIDATION_ERROR
     *                  Content.margin is to high:
     *                      value:
     *                          message: '{"content":[{"margin":"TOO_HIGH"}]}'
     *                          code: VALIDATION_ERROR
     *                  Content.margin not multiple to 250:
     *                      value:
     *                          message: '{"content":[{"margin":"NOT_MULTIPLE_TO 250"}]}'
     *                          code: VALIDATION_ERROR
     *                  Content.margin is not integer:
     *                      value:
     *                          message: '{"content":[{"margin":"NOT_INTEGER"}]}'
     *                          code: VALIDATION_ERROR
     *                  No content.dimension specify or empty:
     *                      value:
     *                          message: '{"content":[{"dimension":"REQUIRED"}]}'
     *                          code: VALIDATION_ERROR
     *                  No allow value content.dimension:
     *                      value:
     *                          message: '{"content":[{"dimension":"NOT_ALLOWED_VALUE"}]}'
     *                          code: VALIDATION_ERROR
     */
    put: makeRequestHandler(
        NetworkLsbScheduleUpdate,
        (req) => ({
            uid: req.params.uid,
            name: req.body.name,
            content: req.body.content,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/network/lsb_schedules/{uid}:
     *   delete:
     *     description: Remove network LSB schedule by uid
     *     tags:
     *       - Network LSB schedule API
     *     parameters:
     *      - in: path
     *        name: uid
     *        schema:
     *          type: string
     *          required: true
     *          example: kYuzyFjYQl65g0MqNvhaYA
     *          description: network LSB schedule unique identifier
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  type: object
     *                  properties:
     *                      uid:
     *                          type: string
     *              example:
     *                  uid: kYuzyFjYQl65g0MqNvhaYA
     *       '404':
     *          description: Network schedule with the specified UID was not found.
     *          content:
     *              application/json:
     *                  schema:
     *                      $ref: '#/components/schemas/NotFoundError'
     *                  examples:
     *                      Non existed network schedule:
     *                          value:
     *                              message: Network schedule with uid ... not found
     *                              code: NOT_FOUND_ERROR
     */
    del: makeRequestHandler(
        NetworkLsbScheduleDelete,
        (req) => ({
            uid: req.params && req.params.uid,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
};
