import makeRequestHandler from '../../../utils/makeRequestHandler.mjs';
import { NetworkSettingGet, NetworkSettingSave, NetworkSettingDelete } from '../../../../usecases/index.mjs';

export default {
    /**
     * @swagger
     * /api/v3/network/settings:
     *   get:
     *     description: Retrieve network settings for all locations
     *     tags:
     *       - Network Settings API
     *     responses:
     *       '200':
     *         description: network settings
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NetworkSettings'
     *              example:
     *                  settings:
     *                      -
     *                          location_uid: _2E2DIF5SAemIHbnIhLbGg
     *                          created_at: 2021-08-25T14:30:00.000Z
     *                          updated_at: 2021-08-25T14:30:00.000Z
     *                          value:
     *                              active_schedule_uid: tGhW9YSET-GoTy2FI5TzsA
     *                              lsb: ON
     *                              masters:
     *                                  -
     *                                      anchor_uid: MASTER_1-GhW9YSET-GoTy
     *                                      sfd: 500
     *                                      auto_sync: true
     *                              distributors:
     *                                  -
     *                                      anchor_uid: DISTRIB_1-GhW9YSET-GoT
     *                                      master_uid: null
     *                                  -
     *                                      anchor_uid: DISTRIB_2-GhW9YSET-GoT
     *                                      master_uid: MASTER_1-GhW9YSET-GoTy
     */
    getAll: makeRequestHandler(
        NetworkSettingGet,
        () => ({}),
        (settings, res) => {
            res.json(settings);
        }
    ),

    /**
     * @swagger
     * /api/v3/network/settings/{location_uid}:
     *   get:
     *     description: Retrieve network setting for a specific location
     *     tags:
     *       - Network Settings API
     *     parameters:
     *      - in: path
     *        name: location_uid
     *        schema:
     *          type: string
     *          required: true
     *          example: _2E2DIF5SAemIHbnIhLbGg
     *          description: location UID
     *     responses:
     *       '200':
     *         description: network settings
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NetworkSettings'
     *              example:
     *                  settings:
     *                      -
     *                          location_uid: _2E2DIF5SAemIHbnIhLbGg
     *                          created_at: 2021-08-25T14:30:00.000Z
     *                          updated_at: 2021-08-25T14:30:00.000Z
     *                          value:
     *                              active_schedule_uid: tGhW9YSET-GoTy2FI5TzsA
     *                              lsb: ON
     *                              masters:
     *                                  -
     *                                      anchor_uid: MASTER_1-GhW9YSET-GoTy
     *                                      sfd: 500
     *                                      auto_sync: true
     *                              distributors:
     *                                  -
     *                                      anchor_uid: DISTRIB_1-GhW9YSET-GoT
     *                                      master_uid: null
     *                                  -
     *                                      anchor_uid: DISTRIB_2-GhW9YSET-GoT
     *                                      master_uid: MASTER_1-GhW9YSET-GoTy
     *       '404':
     *         description: Network settings for location with specified uid not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              example:
     *                  message: Network settings for location with uid ... not found.
     *                  code: NOT_FOUND_ERROR
     */
    get: makeRequestHandler(
        NetworkSettingGet,
        (req) => ({
            location_uid: req.params && req.params.location_uid,
        }),
        (setting, res) => {
            res.json(setting);
        }
    ),

    /**
     * @swagger
     * /api/v3/network/settings:
     *   post:
     *     description: Add network setting
     *     tags:
     *       - Network Settings API
     *     requestBody:
     *        content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NetworkSettingCreateBody'
     *              example:
     *                  location_uid: _2E2DIF5SAemIHbnIhLbGg
     *                  value:
     *                      active_schedule_uid: tGhW9YSET-GoTy2FI5TzsA
     *                      lsb: ON
     *                      masters:
     *                          -
     *                              anchor_uid: MASTER_1-GhW9YSET-GoTy
     *                              sfd: 500
     *                              auto_sync: true
     *                      distributors:
     *                          -
     *                              anchor_uid: DISTRIB_1-GhW9YSET-GoT
     *                              master_uid: null
     *                          -
     *                              anchor_uid: DISTRIB_2-GhW9YSET-GoT
     *                              master_uid: MASTER_1-GhW9YSET-GoTy
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NetworkSetting'
     *              example:
     *                  location_uid: _2E2DIF5SAemIHbnIhLbGg
     *                  created_at: 2021-08-25T14:30:00.000Z
     *                  updated_at: 2021-08-25T14:30:00.000Z
     *                  value:
     *                      active_schedule_uid: tGhW9YSET-GoTy2FI5TzsA
     *                      lsb: ON
     *                      masters:
     *                          -
     *                              anchor_uid: MASTER_1-GhW9YSET-GoTy
     *                              sfd: 500
     *                              auto_sync: true
     *                      distributors:
     *                          -
     *                              anchor_uid: DISTRIB_1-GhW9YSET-GoT
     *                              master_uid: null
     *                              slot: 150
     *                          -
     *                              anchor_uid: DISTRIB_2-GhW9YSET-GoT
     *                              master_uid: MASTER_1-GhW9YSET-GoTy
     *                              slot: 150
     *       '422':
     *         description: Invalid request parameters.
     *         content:
     *          application/json:
     *              schema:
     *                      $ref: '#/components/schemas/ValidationError'
     *              examples:
     *                  No location_uid:
     *                      value:
     *                          message: '{"location_uid":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Empty location_uid:
     *                      value:
     *                          message: '{"location_uid":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  No lsb:
     *                      value:
     *                          message: '{"value":{"lsb":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  Empty lsb:
     *                      value:
     *                          message: '{"value":{"lsb":"CANNOT_BE_EMPTY"}}'
     *                          code: VALIDATION_ERROR
     *                  Wrong lsb value, allowed ['ON', 'OFF', 'on', 'off']:
     *                      value:
     *                          message: '{"value":{"lsb":"NOT_ALLOWED_VALUE"}}'
     *                          code: VALIDATION_ERROR
     *                  Empty active_schedule_uid:
     *                      value:
     *                          message: '{"value":{"active_schedule_uid":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  No masters:
     *                      value:
     *                          message: '{"value":{"masters":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  No masters.anchor_uid:
     *                      value:
     *                          message: '{"value":{"masters":[{"anchor_uid":"REQUIRED"}]}}'
     *                          code: VALIDATION_ERROR
     *                  No masters.sfd:
     *                      value:
     *                          message: '{"value":{"masters":[{"sfd":"REQUIRED"}]}}'
     *                          code: VALIDATION_ERROR
     *                  Wrong masters.sfd value, allowed 250, 500, 1000:
     *                      value:
     *                          message: '{"value":{"masters":[{"sfd":"NOT_ALLOWED_VALUE"}]}}'
     *                          code: VALIDATION_ERROR
     *                  No masters.auto_sync:
     *                      value:
     *                          message: '{"value":{"masters":[{"auto_sync":"REQUIRED"}]}}'
     *                          code: VALIDATION_ERROR
     *                  No distributors:
     *                      value:
     *                          message: '{"value":{"distributors":"REQUIRED"}}'
     *                          code: VALIDATION_ERROR
     *                  No distributors.anchor_uid:
     *                      value:
     *                          message: '{"value":{"distributors":[{"anchor_uid":"REQUIRED"}]}}'
     *                          code: VALIDATION_ERROR
     *                  No distributors.slot:
     *                      value:
     *                          message: '{"value":{"distributors":[{"slot":"REQUIRED"}]}}'
     *                          code: VALIDATION_ERROR
     *                  Distributors.slot should be integer:
     *                      value:
     *                          message: '{"value":{"distributors":[{"slot":"NOT_INTEGER"}]}}'
     *                          code: VALIDATION_ERROR
     *       '404':
     *         description: Location with specified uid not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              example:
     *                  Location not found:
     *                      value:
     *                          message: Location with uid ... not found
     *                          code: NOT_FOUND_ERROR
     */
    post: makeRequestHandler(
        NetworkSettingSave,
        (req) => ({
            ...req.body,
        }),
        (result, res) => {
            res.json(result);
        }
    ),

    /**
     * @swagger
     * /api/v3/network/settings/{location_uid}:
     *   delete:
     *     description: Update network setting
     *     tags:
     *       - Network Settings API
     *     parameters:
     *      - in: path
     *        name: location_uid
     *        schema:
     *          type: string
     *          required: true
     *          example: _2E2DIF5SAemIHbnIhLbGg
     *          description: location UID
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NetworkSetting'
     *              example:
     *                  location_uid: _2E2DIF5SAemIHbnIhLbGg
     *                  created_at: 2021-08-25T14:30:00.000Z
     *                  updated_at: 2021-08-25T14:30:00.000Z
     *                  value:
     *                      active_schedule_uid: tGhW9YSET-GoTy2FI5TzsA
     *                      lsb: ON
     *                      masters:
     *                          -
     *                              anchor_uid: MASTER_1-GhW9YSET-GoTy
     *                              sfd: 500
     *                              auto_sync: true
     *                      distributors:
     *                          -
     *                              anchor_uid: DISTRIB_1-GhW9YSET-GoT
     *                              master_uid: null
     *                          -
     *                              anchor_uid: DISTRIB_2-GhW9YSET-GoT
     *                              master_uid: MASTER_1-GhW9YSET-GoTy
     *       '404':
     *         description: Location with specified uid not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              examples:
     *                  Location not found:
     *                      value:
     *                          message: Location with uid ... not found
     *                          code: NOT_FOUND_ERROR
     *                  Network Setting not found:
     *                      value:
     *                          message: Network settings for location with uid ... not found
     *                          code: NOT_FOUND_ERROR
     */
    del: makeRequestHandler(
        NetworkSettingDelete,
        (req) => ({
            location_uid: req.params.location_uid,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
};
