import makeRequestHandler from '../../../utils/makeRequestHandler.mjs';
import { SublocationGet, SublocationSave, SublocationUpdate, SublocationDelete } from '../../../../usecases/index.mjs';

export default {
    /**
     * @swagger
     * /api/v3/sublocations:
     *   get:
     *     description: Retrieve sublocations list
     *     tags:
     *       - Sublocations API
     *     responses:
     *       '200':
     *         description: sublocations
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/Sublocations'
     *              example:
     *                  sublocations:
     *                      -
     *                          uid: kYuzyFjYQl65g0MqNvhaYA
     *                          location_uid: oVi39JDNTLO9qdeNM-Ecvg
     *                          name: Warehouse 1
     *                          id: 1
     *                          created_at: 2021-04-21T10:00:00.000Z
     *                          updated_at: 2021-04-21T10:00:00.000Z
     *                      -
     *                          uid: kYuzyFjYQlasfd32ZXhdYA
     *                          location_uid: oVi39JDNTLO9qdeNM-Ecvg
     *                          name: Warehouse 2
     *                          id: 1
     *                          created_at: 2021-04-21T10:00:00.000Z
     *                          updated_at: 2021-04-21T10:00:00.000Z
     */
    getAll: makeRequestHandler(
        SublocationGet,
        () => ({}),
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/sublocations/{uid}:
     *   get:
     *     description: Retrieve single sublocation
     *     tags:
     *       - Sublocations API
     *     parameters:
     *      - in: path
     *        name: uid
     *        schema:
     *          type: string
     *          required: true
     *          example: kYuzyFjYQl65g0MqNvhaYA
     *          description: sublocation unique identifier
     *     responses:
     *       '200':
     *         description: sublocations
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/Sublocations'
     *              example:
     *                  sublocations:
     *                      -
     *                          uid: kYuzyFjYQl65g0MqNvhaYA
     *                          location_uid: oVi39JDNTLO9qdeNM-Ecvg
     *                          name: Warehouse 1
     *                          id: 1
     *                          created_at: 2021-04-21T10:00:00.000Z
     *                          updated_at: 2021-04-21T10:00:00.000Z
     *       '404':
     *         description: Sublocation with the specified UID was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              example:
     *                  message: Sublocation with uid ... not found
     *                  code: NOT_FOUND_ERROR
     */
    get: makeRequestHandler(
        SublocationGet,
        (req) => ({
            uid: req.params.uid,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/sublocations:
     *   post:
     *     description: Create sublocation
     *     tags:
     *       - Sublocations API
     *     requestBody:
     *      content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/SublocationCreateBody'
     *              example:
     *                  name: Warehouse 1
     *                  elevation: 0
     *                  location_uid: oVi39JDNTLO9qdeNM-Ecvg
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/Sublocation'
     *              example:
     *                  uid: kYuzyFjYQl65g0MqNvhaYA
     *                  location_uid: oVi39JDNTLO9qdeNM-Ecvg
     *                  name: Warehouse 1
     *                  id: 1
     *                  created_at: 2021-04-21T10:00:00.000Z
     *                  updated_at: 2021-04-21T10:00:00.000Z
     *       '422':
     *         description: Invalid request parameters.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/ValidationError'
     *              examples:
     *                  No name specified:
     *                      value:
     *                          message: '{"name":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Empty name:
     *                      value:
     *                          message: '{"name":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  No location_uid specified:
     *                      value:
     *                          message: '{"location_uid":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Empty location_uid:
     *                      value:
     *                          message: '{"location_uid":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  Elevation not multiply to 10:
     *                      value:
     *                          message: '{"elevation":"NOT_MULTIPLE_TO 10"}'
     *                          code: VALIDATION_ERROR
     *                  Elevation is not integer:
     *                      value:
     *                          message: '{"elevation":"NOT_INTEGER"}'
     *                          code: VALIDATION_ERROR
     *       '404':
     *         description: Location with the specified UID was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              example:
     *                  message: Location with uid ... not found
     *                  code: NOT_FOUND_ERROR
     */
    post: makeRequestHandler(
        SublocationSave,
        (req) => ({
            name: req.body.name,
            location_uid: req.body.location_uid,
            elevation: req.body.elevation,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/sublocations/{uid}:
     *   put:
     *     description: Update sublocation
     *     tags:
     *       - Sublocations API
     *     parameters:
     *      - in: path
     *        name: uid
     *        schema:
     *          type: string
     *          required: true
     *          example: eJEG2Q2kRnSHJaBdHxQ3Pw
     *          description: sublocation unique identifier
     *     requestBody:
     *      content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/SublocationUpdateBody'
     *              example:
     *                  name: Warehouse 2
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/Sublocation'
     *              example:
     *                  uid: kYuzyFjYQl65g0MqNvhaYA
     *                  location_uid: oVi39JDNTLO9qdeNM-Ecvg
     *                  name: Warehouse 2
     *                  id: 1
     *                  created_at: 2021-04-21T10:00:00.000Z
     *                  updated_at: 2021-04-21T10:00:00.000Z
     *       '404':
     *         description: Sublocation with the specified UID was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              example:
     *                  message: Sublocation with uid ... not found
     *                  code: NOT_FOUND_ERROR
     *       '422':
     *         description: Invalid request parameters.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/ValidationError'
     *              examples:
     *                  Empty name:
     *                      value:
     *                          message: '{"name":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  Elevation not multiply to 10:
     *                      value:
     *                          message: '{"elevation":"NOT_MULTIPLE_TO 10"}'
     *                          code: VALIDATION_ERROR
     *                  Elevation is not integer:
     *                      value:
     *                          message: '{"elevation":"NOT_INTEGER"}'
     *                          code: VALIDATION_ERROR
     */
    put: makeRequestHandler(
        SublocationUpdate,
        (req) => ({
            uid: req.params.uid,
            name: req.body.name,
            elevation: req.body.elevation,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/sublocations/{uid}:
     *   delete:
     *     description: Remove sublocation by uid
     *     tags:
     *       - Sublocations API
     *     parameters:
     *      - in: path
     *        name: uid
     *        schema:
     *          type: string
     *          required: true
     *          example: kYuzyFjYQl65g0MqNvhaYA
     *          description: sublocation unique identifier
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  type: object
     *                  properties:
     *                      uid:
     *                          type: string
     *              example:
     *                  uid: kYuzyFjYQl65g0MqNvhaYA
     *       '404':
     *         description: Sublocation with the specified UID was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              example:
     *                  message: Sublocation with uid ... not found
     *                  code: NOT_FOUND_ERROR
     */
    del: makeRequestHandler(
        SublocationDelete,
        (req) => ({
            uid: req.params && req.params.uid,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
};
