import makeRequestHandler from '../../../utils/makeRequestHandler.mjs';
import { TagAssociationGet, TagAssociationSave, TagAssociationClose } from '../../../../usecases/index.mjs';

export default {
    /**
     * @swagger
     * /api/v3/tag_assignments:
     *   get:
     *     description: Retrieve tag assignments list
     *     tags:
     *       - Tag Assignment API
     *     responses:
     *       '200':
     *         description: tag assignments
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/TagAssociations'
     *              example:
     *                  tag_assignments:
     *                      -
     *                          trackable_uid: 28bOWPAMSgilkQEb-2wLqg
     *                          mac: E4956EA5010B
     *                          created_at: 2021-04-21T10:00:00.000Z
     *                      -
     *                          trackable_uid: 28bOWPAMSgilkQE432dsSd
     *                          mac: E4956EA5010A
     *                          created_at: 2021-04-21T10:00:00.000Z
     */
    getAll: makeRequestHandler(
        TagAssociationGet,
        () => ({}),
        (tagAssociations, res) => {
            res.json({ tag_assignments: tagAssociations });
        }
    ),

    /**
     * @swagger
     * /api/v3/tag_assignments:
     *   post:
     *     description: Create tag assignment
     *     tags:
     *       - Tag Assignment API
     *     requestBody:
     *        content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/TagAssignmentCreateBody'
     *              example:
     *                  mac: E4956EA5010B
     *                  trackable_uid: 28bOWPAMSgilkQEb-2wLqg
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/TagAssociation'
     *              example:
     *                  trackable_uid: 28bOWPAMSgilkQEb-2wLqg
     *                  mac: E4956EA5010B
     *                  created_at: 2021-04-21T10:00:00.000Z
     *       '422':
     *          description: Invalid request parameters.
     *          content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/ValidationError'
     *              examples:
     *                  No mac:
     *                      value:
     *                          message: '{"mac":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Empty mac:
     *                      value:
     *                          message: '{"mac":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  Wrong MAC format:
     *                      value:
     *                          message: '{"mac":"WRONG_MAC_FORMAT"}'
     *                          code: VALIDATION_ERROR
     *                  No trackable_uid:
     *                      value:
     *                          message: '{"trackable_uid":"REQUIRED"}'
     *                          code: VALIDATION_ERROR
     *                  Empty trackable_uid:
     *                      value:
     *                          message: '{"trackable_uid":"CANNOT_BE_EMPTY"}'
     *                          code: VALIDATION_ERROR
     *                  Node assignment exist:
     *                      value:
     *                          message: 'Assignment with tag ... already exist'
     *                          code: VALIDATION_ERROR
     *                  Trackable assignment exist:
     *                      value:
     *                          message: 'Assignment with trackable ... already exist'
     *                          code: VALIDATION_ERROR
     *       '404':
     *         description: Tag or trackable with the specified parameters was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              examples:
     *                  Node not found:
     *                      value:
     *                          message: Tag with mac ... not found
     *                          code: NOT_FOUND_ERROR
     *                  Trackable not found:
     *                      value:
     *                          message: Trackable with uid ... not found
     *                          code: NOT_FOUND_ERROR
     */
    post: makeRequestHandler(
        TagAssociationSave,
        (req) => ({
            mac: req.body.mac,
            trackable_uid: req.body.trackable_uid,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/tag_assignments:
     *   put:
     *     description: Remove tag assignment
     *     tags:
     *       - Tag Assignment API
     *     requestBody:
     *        content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/TagAssignmentCloseBody'
     *              example:
     *                  mac: E4956EA5010B
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/TagAssociation'
     *              example:
     *                  trackable_uid: 28bOWPAMSgilkQEb-2wLqg
     *                  mac: E4956EA5010B
     *                  created_at: 2021-04-21T10:00:00.000Z
     *       '422':
     *          description: Invalid request parameters.
     *          content:
     *              application/json:
     *                  schema:
     *                      $ref: '#/components/schemas/ValidationError'
     *                  examples:
     *                      No mac or trackable uid:
     *                          value:
     *                              message: '{"mac":"REQUIRED","trackable_uid":"REQUIRED"}'
     *                              code: VALIDATION_ERROR
     *                      Empty mac:
     *                          value:
     *                              message: '{"mac":"CANNOT_BE_EMPTY"}'
     *                              code: VALIDATION_ERROR
     *                      Empty trackable_uid:
     *                          value:
     *                              message: '{"trackable_uid":"CANNOT_BE_EMPTY"}'
     *                              code: VALIDATION_ERROR
     *                      Wrong MAC format:
     *                          value:
     *                              message: '{"mac":"WRONG_MAC_FORMAT"}'
     *                              code: VALIDATION_ERROR
     *       '404':
     *         description: Tag assignment with the specified mac address was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              examples:
     *                  Wrong mac and anchor uid:
     *                      message: Assignment with tag ... and trackable ... not found
     *                      code: NOT_FOUND_ERROR
     *                  Wrong mac:
     *                      message: Assignment with tag ... not found
     *                      code: NOT_FOUND_ERROR
     *                  Wrong anchor uid:
     *                      message: Assignment with trackable ... not found
     *                      code: NOT_FOUND_ERROR
     */
    put: makeRequestHandler(
        TagAssociationClose,
        (req) => ({
            mac: req.body && req.body.mac,
            trackable_uid: req.body && req.body.trackable_uid,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
};
