import makeRequestHandler from '../../../utils/makeRequestHandler.mjs';
import { TagDelete, TagSave, TagGet } from '../../../../usecases/index.mjs';

export default {
    /**
     * @swagger
     * /api/v3/tags:
     *   get:
     *     description: Retrieve tags list
     *     tags:
     *       - Tags API
     *     responses:
     *       '200':
     *         description: tags
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/Tags'
     *              example:
     *                  tags:
     *                      -
     *                          mac: E4956EA5010A
     *                          last_heard: 2021-04-21T10:00:00.000Z
     *                          node_id: 1
     *                          cbid: null
     *                          created_at: 2021-04-21T10:00:00.000Z
     *                      -
     *                          mac: E4956EA5010B
     *                          last_heard: 2021-04-21T10:00:00.000Z
     *                          node_id: 2
     *                          cbid: null
     *                          created_at: 2021-04-21T10:00:00.000Z
     */
    getAll: makeRequestHandler(
        TagGet,
        (req) => ({
            mac: req.query && req.query.mac,
        }),
        (tags, res) => {
            res.json({ tags });
        }
    ),

    /**
     * @swagger
     * /api/v3/tags/{mac}:
     *   get:
     *     description: Retrieve tags list
     *     tags:
     *       - Tags API
     *     parameters:
     *      - in: path
     *        name: mac
     *        schema:
     *          type: string
     *          required: true
     *          example: e4956ea5010a
     *          description: tag MAC address
     *     responses:
     *       '200':
     *         description: tags
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/Tags'
     *              example:
     *                  tags:
     *                      -
     *                          mac: E4956EA5010A
     *                          last_heard: 2021-04-21T10:00:00.000Z
     *                          node_id: 1
     *                          cbid: null
     *                          created_at: 2021-04-21T10:00:00.000Z
     *       '404':
     *         description: A tag with the specified MAC was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              example:
     *                  message: Tag with MAC ... not found
     *                  code: NOT_FOUND_ERROR
     *       '422':
     *         description: Wrong format of the tag MAC address.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/ValidationError'
     *              example:
     *                  message: '{"mac":"WRONG_MAC_FORMAT"}'
     *                  code: VALIDATION_ERROR
     */
    get: makeRequestHandler(
        TagGet,
        (req) => ({
            mac: req.params && req.params.mac,
        }),
        (tags, res) => {
            res.json({ tags });
        }
    ),
    /**
     * @swagger
     * /api/v3/tags:
     *   post:
     *     description: Create tag
     *     tags:
     *       - Tags API
     *     requestBody:
     *        content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/TagCreateBody'
     *              example:
     *                  mac: e4956ea5010a
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/Tag'
     *              example:
     *                  mac: E4956EA5010A
     *                  last_heard: 2021-04-21T10:00:00.000Z
     *                  node_id: 1
     *                  cbid: null
     *                  created_at: 2021-04-21T10:00:00.000Z
     *       '422':
     *          description: Invalid request parameters.
     *          content:
     *              application/json:
     *                  schema:
     *                      $ref: '#/components/schemas/ValidationError'
     *                  examples:
     *                      No MAC specified:
     *                          value:
     *                              message: '{"mac":"REQUIRED"}'
     *                              code: VALIDATION_ERROR
     *                      Empty mac:
     *                          value:
     *                              message: '{"mac":"CANNOT_BE_EMPTY"}'
     *                              code: VALIDATION_ERROR
     *                      Wrong MAC format:
     *                          value:
     *                              message: '{"mac":"WRONG_MAC_FORMAT"}'
     *                              code: VALIDATION_ERROR
     *                      MAC not available:
     *                          value:
     *                              message: Tag with MAC ... already exist
     *                              code: VALIDATION_ERROR
     */
    post: makeRequestHandler(
        TagSave,
        (req) => ({
            mac: req.body.mac,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
    /**
     * @swagger
     * /api/v3/tags/{mac}:
     *   delete:
     *     description: Remove tag by MAC address
     *     tags:
     *       - Tags API
     *     parameters:
     *      - in: path
     *        name: mac
     *        schema:
     *          type: string
     *          required: true
     *          example: e4956ea5010a
     *          description: hardware MAC address 12 characters length (6 bytes heximal value).
     *     responses:
     *       '200':
     *         content:
     *          application/json:
     *              schema:
     *                  type: object
     *                  properties:
     *                      mac:
     *                          type: string
     *              example:
     *                  mac: e4956ea5010a
     *       '404':
     *         description: A tag with the specified MAC was not found.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/NotFoundError'
     *              example:
     *                  message: Tag with MAC ... not found
     *                  code: NOT_FOUND_ERROR
     *       '422':
     *         description: Wrong format of the tag MAC address.
     *         content:
     *          application/json:
     *              schema:
     *                  $ref: '#/components/schemas/ValidationError'
     *              example:
     *                  message: '{"mac":"WRONG_MAC_FORMAT"}'
     *                  code: VALIDATION_ERROR
     */
    del: makeRequestHandler(
        TagDelete,
        (req) => ({
            mac: req.params && req.params.mac,
        }),
        (result, res) => {
            res.json(result);
        }
    ),
};
