import ModelBase from '../ModelBase.mjs';
import { toISOTime } from '../../utils/index.mjs';
import { NodeId } from '../id/index.mjs';

export class Node extends ModelBase {
    /**
     *
     * @param node {SolutionNode}
     * @param platformItems {Array<PlatformNode>}
     * @returns {{uid: null, last_heard: (string|null), updated_at: (string|null), cbid: (string|null), name: null, created_at: (string|null), type: (undefined|*), mac: (*|string|null)}}
     */
    static formatSolutionNode(node, platformItems) {
        const mac = (node.mac_address && node.mac_address.toUpperCase()) || null;
        const parameters = node.parameters ? JSON.parse(node.parameters): null
        const platform_node = platformItems.find(x => x.mac === mac);
        return {
            uid: platform_node ? platform_node.uid : node.uid,
            mac,
            last_heard: toISOTime(node.last_heard) || null,
            created_at: toISOTime(node.created_at) || null,
            updated_at: toISOTime(node.updated_at || node.created_at) || null,
            // node_id: await repository.nodeId.nextValue() || null,
            type: node.node_type === 'CP' ? undefined : node.node_type,
            cbid: (parameters && parameters.hw_info && parameters.hw_info.cbid) || null,
        }
    }
    
    /**
     *
     * @param node {PlatformNode}
     * @returns {PlatformNode}
     */
    static formatPlatformNode(node) {
        return {
            uid: node.uid || null,
            mac: node.mac || null,
            last_heard: (node.last_heard) || null,
            created_at: (node.created_at) || null,
            updated_at: (node.updated_at) || null,
            // node_id: await repository.nodeId.nextValue() || null,
            type: node.type || undefined,
            cbid: node.cbid || null,
        }
    }
    
    /**
     *
     * @param platformItems {Array<PlatformNode>}
     * @returns {Promise<SolutionNode[]>}
     */
    async collectSolutionEntities(platformItems) {
        return (await this.solutionRepositoryInstance.node.readAll())
            .map(solutionNode => Node.formatSolutionNode(solutionNode, platformItems));
    }
    
    /**
     *
     * @returns {Promise<PlatformNode[]>}
     */
    async collectPlatformEntities() {
        return (await this.platformRepositoryInstance.node.findAll()).map(Node.formatPlatformNode)
    }
    
    /**
     *
     * @param items {PlatformNode[]}
     * @returns {Promise<void>}
     */
    async addToPlatform(items) {
        for (const item of items) {
            item.node_id = Number(await this.platformRepositoryInstance.nodeId.nextValue());
            await this.platformRepositoryInstance.node.save(item);
        }
    }
    
    /**
     *
     * @param items {PlatformNode[]}
     * @returns {Promise<void>}
     */
    async removeInPlatform(items) {
        for (const item of items) {
            const node = await this.platformRepositoryInstance.node.findByMac(item.mac);
            if (node) {
                await this.platformRepositoryInstance.node.del(node);
                await new NodeId().remove(node.node_id);
            }
        }
    }
    
    /**
     *
     * @param items {PlatformNode[]}
     * @returns {Promise<void>}
     */
    async updateInPlatform(items) {
        for (const item of items) {
            await this.platformRepositoryInstance.node.save(item);
        }
    }
}
