import RepoRedisBase from '../RepoRedisBase.mjs';
import { RepoError } from '../models/index.mjs';
import flat from 'flat';

const flatten = flat.flatten;
const unflatten = flat.unflatten;

export class NetworkLsbScheduleRepository extends RepoRedisBase {
    /**
     * @typedef {Class} NetworkLsbScheduleRepository
     */

    /**
     * @param {Object} options - repository options
     * @param {Object} options.repositoryInstance - database connection
     * @param {String} options.prefix - database prefix
     */
    constructor(options) {
        const { prefix, repositoryInstance } = options;

        super(repositoryInstance, `${prefix}:lsb_schedule`);
    }

    /**
     * @method
     * @return {Promise<Array<Object>>} returns entities list
     */
    async findAll() {
        const result = [];
        try {
            const keys = await this.scanAll('0', `${this.document}:uid:*`, new Set());
            for (const key of keys) {
                const obj = await this.getByKey(key);
                if (obj) {
                    result.push(obj);
                }
            }
            return result;
        } catch (e) {
            throw new RepoError(e);
        }
    }

    /**
     * @method
     * @return {Promise<Array<Object>>} returns entities list
     */
    async findByLocationUid(locationUid) {
        const result = [];
        try {
            const scheduleUidArray = await this.repoInstance.smembers(`${this.document}:loc_uid:${locationUid}`);
            if (scheduleUidArray.length === 0) {
                return result;
            }
            const keys = await this.scanAll('0', `${this.document}:uid:*`, new Set());
            for (const key of keys) {
                const obj = await this.getByKey(key);
                if (obj && obj.location_uid && scheduleUidArray.includes(obj.uid)) {
                    result.push(obj);
                }
            }
            return result;
        } catch (e) {
            throw new RepoError(e);
        }
    }

    /**
     * @method
     * @param {String} uid - entity unique identifier
     * @return {Promise<Object>} returns result of search by key
     */
    async findByUid(uid) {
        const flatten_obj = await this.repoInstance.hgetall(`${this.document}:uid:${uid}`);
        if (!flatten_obj) {
            return null;
        }
        return unflatten(flatten_obj);
    }

    /**
     * @method
     * @param {Object} lsb_schedule - data to save in DB
     * @return {Promise<Object>} returns data saved in DB
     */
    async save(lsb_schedule) {
        if (!this.document) {
            throw new RepoError(`save: no document`);
        }
        if (!lsb_schedule || !lsb_schedule.uid || typeof lsb_schedule !== 'object') {
            throw new RepoError(`save: object saving is aborted due to object format: ${JSON.stringify(lsb_schedule)}`);
        }
        const fields = [];
        for (const [key, value] of Object.entries(flatten(lsb_schedule))) {
            if (value || typeof value === 'number') {
                // do not save empty values
                fields.push(key, value);
            }
        }

        await this.repoInstance.hmset(`${this.document}:uid:${lsb_schedule.uid}`, ...fields);
        if (lsb_schedule.location_uid) {
            await this.repoInstance.sadd(`${this.document}:loc_uid:${lsb_schedule.location_uid}`, lsb_schedule.uid);
        }
        return lsb_schedule;
    }

    /**
     *
     * @param {Object} lsb_schedule
     * @property {String} lsb_schedule.uid
     * @property {String} lsb_schedule.location_uid
     * @returns {Promise<void>}
     */
    async del(lsb_schedule) {
        if (!this.document) {
            throw new RepoError(`save: no document`);
        }
        if (!lsb_schedule || !lsb_schedule.uid || typeof lsb_schedule !== 'object') {
            throw new RepoError(`del: object removal is aborted due to object format: ${lsb_schedule}`);
        }

        await this.repoInstance.del(`${this.document}:uid:${lsb_schedule.uid}`);
        await this.repoInstance.srem(`${this.document}:loc_uid:${lsb_schedule.location_uid}`, lsb_schedule.uid);
    }
}
